require 'runit/testcase'

require 'kagemai/reporttype'
require 'kagemai/report'
require 'kagemai/message'

do_pg_test = false
begin
  require 'postgres'
  require 'kagemai/pgstore'
  do_pg_test = true
rescue LoadError
end

if do_pg_test then
  Kagemai::Config[:enable_postgres] = true
  include Kagemai

  class TestPostgresStore < RUNIT::TestCase
    def setup()
      sleep(0)
      Kagemai::MessageBundle.open('resource', 'ja', 'messages')

      @project_id = 'kagemai_unittest'
      @report_type = ReportType.load('test/testfile/rtype3.xml')
      @charset = 'EUC-JP'

      @email = ['fukouka@daifukuya.com', 'tomoyuki@daifukuya.com']
      @body = ['hell world', 'hello dbi']
      @opt = [{'o1' => 'v1', 'o2' => true}, {'o3' => false}]

      @messages = []
      0.upto(1) do |i|
        msg = Message.new(@report_type) 
        msg['Email'] = @email[i] 
        msg['message'] = @body[i]
        @opt[i].each{|k, v| msg.set_option(k, v)}
        @messages << msg
      end

      PostgresStore.create(nil, @project_id, @report_type, @charset)
      @store = PostgresStore.new(nil, @project_id, @report_type, @charset)
    end

    def teardown()
      sleep(1)
      PostgresStore.destroy(nil, @project_id)
      sleep(2)
    end

    def test_nextid()
      assert_equals(1, @store.next_id())
    end

    def test_store()
      report_id = nil
      
      @store.transaction {
        report_id = @store.next_id
        report = Report.new(@report_type, report_id)
        report.add_message(@messages[0])
        report.add_message(@messages[1])
        @store.store(report)
      }

      report_rows = nil
      message_rows = nil
      @store.execute() do |db|
        report_rows = db.select_all('select * from reports')
        message_rows = db.select_all('select * from messages')
      end
      assert_equals(1, report_rows.size)
      assert_equals(2, message_rows.size)

      assert_equals(1, report_rows[0]['id'])

      count = 0
      0.upto(1) do |i|
        result = message_rows[i]
        msg = @messages[i]
        assert_equals(report_id, result['report_id'])
        assert_equals(count + 1, result['id'])
        assert_equals(msg['Email'], result['email'])
        assert_equals(msg['message'], result['message'])
        count += 1
      end
    end
    
    def test_load
      report_id = nil
      report = nil
      @store.transaction {
        report_id = @store.next_id
        report = Report.new(@report_type, report_id)
        report.add_message(@messages[0])
        report.add_message(@messages[1])
        @store.store(report)
      }
      time = report.first.time
      
      report = @store.load(@report_type, report_id)
      assert_instance_of(Kagemai::Report, report)
      assert_equals(2, report.size)
      assert_equals(time.to_i, report.first.time.to_i)

      report.each_with_index do |message, i|
        assert_equals(i + 1, message.id)
        assert_equals(@messages[i]['Email'], message['Email'])
        assert_equals(@messages[i]['message'], message['message'])

        opt = {}
        message.each_option{|k ,v| opt[k] = v}
        assert_equals(@opt[i], opt)
      end
    end

    def test_load2
      assert_exception(Kagemai::ParameterError) {
        @store.load(@report_type, 100)
      }
    end

    def test_update()
      report = nil
      report_id = nil
      
      message = @messages[0].dup
      @store.transaction {
        report_id = @store.next_id
        report = Report.new(@report_type, report_id)
        report.add_message(@messages[0])
        report.add_message(@messages[1])
        @store.store(report)
      }

      report.last['Email'] = 'kagemai_test@daifukuya.com'
      report.last.modified = true
      @store.transaction {
        @store.update(report)
      }
      
      report_rows = nil
      message_rows = nil
      @store.execute() do |db|
        report_rows = db.select_all('select * from reports order by id')
        message_rows = db.select_all('select * from messages order by id')
      end
      assert_equals(1, report_rows.size)
      assert_equals(2, message_rows.size)

      assert_equals(1, report_rows[0]['id'])

      email = [@email[0], 'kagemai_test@daifukuya.com']

      count = 0
      0.upto(1) do |i|
        result = message_rows[i]
        msg = @messages[i]
        assert_equals(report_id, result['report_id'])
        assert_equals(email[i], result['email'])
        assert_equals(@body[i], result['message'])
        count += 1
      end
    end

    def test_update2()
      report = nil
      report_id = nil
      
      message = @messages[0].dup
      @store.transaction {
        report_id = @store.next_id
        report = Report.new(@report_type, report_id)
        report.add_message(@messages[0])
        report.add_message(@messages[1])
        @store.store(report)
      }
      lreport = @store.load(@report_type, report_id)
      assert_equals(2, lreport.size)
      assert_equals(@email[1], lreport.last['Email'])

      @store.transaction {
        report = Report.new(@report_type, report_id)
        report.add_message(@messages[0])
        assert_equals(1, report.size)
        @store.update(report)
      }
      lreport = @store.load(@report_type, report_id)
      assert_equals(1, lreport.size)
      assert_equals(@email[0], lreport.last['Email'])
    end

    def test_capital_project()
      project_id = 'PGStoreTestProject'
      
      PostgresStore.create(nil, project_id, @report_type, @charset)
      store = PostgresStore.new(nil, project_id, @report_type, @charset)
      sleep(1)
      PostgresStore.destroy(nil, project_id)
    end

  end
end
